@@ -12,7 +12,6 @@
 
 from __future__ import division
 
-import sys
 import warnings
 from abc import ABCMeta, abstractmethod
 
@@ -25,12 +24,8 @@ import furl
 from furl.omdict1D import omdict1D
 from furl.compat import OrderedDict as odict
 
-PYTHON_27PLUS = sys.version_info >= (2, 7)
+import unittest
 
-if PYTHON_27PLUS:
-    import unittest
-else:
-    import unittest2 as unittest
 
 #
 # TODO(grun): Add tests for furl objects with strict=True. Make sure
@@ -240,15 +235,17 @@ class TestPath(unittest.TestCase):
 
     def _test_set_load(self, path_set_or_load):
         p = furl.Path('a/b/c/')
+        assert path_set_or_load(p, furl.Path('asdf/asdf/')) == p
+        assert not p.isabsolute and str(p) == 'asdf/asdf/'
+
         assert path_set_or_load(p, 'asdf/asdf/') == p
-        assert not p.isabsolute
-        assert str(p) == 'asdf/asdf/'
+        assert not p.isabsolute and str(p) == 'asdf/asdf/'
+
         assert path_set_or_load(p, ['a', 'b', 'c', '']) == p
-        assert not p.isabsolute
-        assert str(p) == 'a/b/c/'
+        assert not p.isabsolute and str(p) == 'a/b/c/'
+
         assert path_set_or_load(p, ['', 'a', 'b', 'c', '']) == p
-        assert p.isabsolute
-        assert str(p) == '/a/b/c/'
+        assert p.isabsolute and str(p) == '/a/b/c/'
 
     def test_add(self):
         # URL paths.
@@ -519,17 +516,37 @@ class TestPath(unittest.TestCase):
             p = furl.Path(path)
             assert str(p) == path_encoded
 
+    def test_itruediv(self):
+        p = furl.Path()
+
+        p /= 'a'
+        assert str(p) == 'a'
+
+        p /= 'b'
+        assert str(p) == 'a/b'
+
+        p /= 'c d/'
+        assert str(p) == 'a/b/c%20d/'
+
+        p /= furl.Path('e')
+        assert str(p) == 'a/b/c%20d/e'
+
     def test_truediv(self):
         p = furl.Path()
 
-        p = p / 'a'
-        assert str(p) == 'a'
+        p1 = p / 'a'
+        assert p1 is not p
+        assert str(p1) == 'a'
 
-        p = p / 'b'
-        assert str(p) == 'a/b'
+        p2 = p / 'a' / 'b'
+        assert p2 is not p
+        assert str(p) == ''
+        assert str(p2) == 'a/b'
 
-        p = p / 'c' / 'd e/'
-        assert str(p) == 'a/b/c/d%20e/'
+        # Path objects should be joinable with other Path objects.
+        p3 = furl.Path('e')
+        p4 = furl.Path('f')
+        assert p3 / p4 == furl.Path('e/f')
 
     def test_asdict(self):
         segments = ['wiki', 'ロリポップ']
@@ -1137,6 +1154,12 @@ class TestFurl(unittest.TestCase):
         items = parse_qsl(urlsplit(url).query, True)
         return (key, val) in items
 
+    def test_constructor_and_set(self):
+        f = furl.furl(
+            'http://user:pass@pumps.ru/', args={'hi': 'bye'},
+            scheme='scrip', path='prorp', host='horp', fragment='fraggg')
+        assert f.url == 'scrip://user:pass@horp/prorp?hi=bye#fraggg'
+
     def test_none(self):
         f = furl.furl(None)
         assert str(f) == ''
@@ -1483,17 +1506,30 @@ class TestFurl(unittest.TestCase):
         f.host = 'ohay.com'
         assert str(f) == 'sup://ohay.com/hay%20supppp?space=1+2#sup'
 
+    def test_path_itruediv(self):
+        f = furl.furl('http://www.pumps.com/')
+
+        f /= 'a'
+        assert f.url == 'http://www.pumps.com/a'
+
+        f /= 'b'
+        assert f.url == 'http://www.pumps.com/a/b'
+
+        f /= 'c d/'
+        assert f.url == 'http://www.pumps.com/a/b/c%20d/'
+
     def test_path_truediv(self):
         f = furl.furl('http://www.pumps.com/')
 
-        f = f / 'a'
-        assert f.url == 'http://www.pumps.com/a'
+        f1 = f / 'a'
+        assert f.url == 'http://www.pumps.com/'
+        assert f1.url == 'http://www.pumps.com/a'
 
-        f = f / 'b'
-        assert f.url == 'http://www.pumps.com/a/b'
+        f2 = f / 'c' / 'd e/'
+        assert f2.url == 'http://www.pumps.com/c/d%20e/'
 
-        f = f / 'c' / 'd e/'
-        assert f.url == 'http://www.pumps.com/a/b/c/d%20e/'
+        f3 = f / furl.Path('f')
+        assert f3.url == 'http://www.pumps.com/f'
 
     def test_odd_urls(self):
         # Empty.
@@ -1604,15 +1640,12 @@ class TestFurl(unittest.TestCase):
         # exception on invalid IPv6 addresses.
         furl.furl('http://[0:0:0:0:0:0:0:1:1:1:1:1:1:1:1:9999999999999]/')
 
-        # Malformed IPv6 should raise an exception because
-        # urlparse.urlsplit() raises an exception in Python v2.7+. In
-        # Python <= 2.6, urlsplit() doesn't raise a ValueError on
-        # malformed IPv6 addresses.
-        if PYTHON_27PLUS:
-            with self.assertRaises(ValueError):
-                furl.furl('http://[0:0:0:0:0:0:0:1/')
-            with self.assertRaises(ValueError):
-                furl.furl('http://0:0:0:0:0:0:0:1]/')
+        # Malformed IPv6 should raise an exception because urlparse.urlsplit()
+        # raises an exception on malformed IPv6 addresses.
+        with self.assertRaises(ValueError):
+            furl.furl('http://[0:0:0:0:0:0:0:1/')
+        with self.assertRaises(ValueError):
+            furl.furl('http://0:0:0:0:0:0:0:1]/')
 
         # Invalid host strings should raise ValueError.
         invalid_hosts = ['.', '..', 'a..b', '.a.b', '.a.b.', '$', 'a$b']
@@ -1638,12 +1671,11 @@ class TestFurl(unittest.TestCase):
         assert f.port == 888
 
         # Malformed IPv6 should raise an exception because
-        # urlparse.urlsplit() raises an exception in Python v2.7+.
-        if PYTHON_27PLUS:
-            with self.assertRaises(ValueError):
-                f.netloc = '[0:0:0:0:0:0:0:1'
-            with self.assertRaises(ValueError):
-                f.netloc = '0:0:0:0:0:0:0:1]'
+        # urlparse.urlsplit() raises an exception
+        with self.assertRaises(ValueError):
+            f.netloc = '[0:0:0:0:0:0:0:1'
+        with self.assertRaises(ValueError):
+            f.netloc = '0:0:0:0:0:0:0:1]'
 
         # Invalid ports should raise an exception.
         with self.assertRaises(ValueError):
@@ -1675,13 +1707,12 @@ class TestFurl(unittest.TestCase):
         f.origin = 'ssh://horse-machine.de'
         assert f.url == 'ssh://user:pass@horse-machine.de'
 
-        # Malformed IPv6 should raise an exception because
-        # urlparse.urlsplit() raises an exception in Python v2.7+.
-        if PYTHON_27PLUS:
-            with self.assertRaises(ValueError):
-                f.origin = '[0:0:0:0:0:0:0:1'
-            with self.assertRaises(ValueError):
-                f.origin = 'http://0:0:0:0:0:0:0:1]'
+        # Malformed IPv6 should raise an exception because urlparse.urlsplit()
+        # raises an exception.
+        with self.assertRaises(ValueError):
+            f.origin = '[0:0:0:0:0:0:0:1'
+        with self.assertRaises(ValueError):
+            f.origin = 'http://0:0:0:0:0:0:0:1]'
 
         # Invalid ports should raise an exception.
         with self.assertRaises(ValueError):
@@ -1985,6 +2016,15 @@ class TestFurl(unittest.TestCase):
                '#uwantpump?')
         assert f.url == 'http://pepp.ru/a/b/c#uwantpump?'
 
+        # In edge cases (e.g. URLs without an authority/netloc), behave
+        # identically to urllib.parse.urljoin().
+        f = furl.furl('wss://slrp.com/').join('foo:1')
+        assert f.url == 'wss://slrp.com/foo:1'
+        f = furl.furl('wss://slrp.com/').join('foo:1:rip')
+        assert f.url == 'foo:1:rip'
+        f = furl.furl('scheme:path').join('foo:blah')
+        assert f.url == 'foo:blah'
+
     def test_tostr(self):
         f = furl.furl('http://blast.off/?a+b=c+d&two%20tap=cat%20nap%24')
         assert f.tostr() == f.url
